@@ -22,8 +22,6 @@
   TODO:
 
    * maybe important, requires deeper code changes:
-   - add config setting: use any connection/WLAN only
-     - we don't use bearer information yet
    - download another file with DHCP/WLAN client counters
      - better get them merged into the same file
      - download/processing function evolved from synchronous
@@ -309,9 +307,21 @@ load_settings(CCCHHDoorIsDesktopWidget *self)
 		g_debug(WIDGET_NAME": load_settings: force_connection = %d", self->force_connection);
 	}
 
+	boolval = gconf_client_get_bool(self->gconf_client,
+		GCONF_DIR"/conn_wlan_only", &error);
+
+	if (error) {
+		g_warning(WIDGET_NAME": failed to get configuration value conn_wlan_only: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else {
+		self->conn_wlan_only = boolval;
+		g_debug(WIDGET_NAME": load_settings: conn_wlan_only = %d", self->conn_wlan_only);
+	}
 
 	// auto_update: new since 0.1.10
-	// client_get_bool would return FALSE if key unset
+	// client_get_bool would return FALSE if key unset but default configuration is TRUE
 	GConfValue *confval = gconf_client_get_without_default(self->gconf_client,
 		GCONF_DIR"/auto_update", &error);
 
@@ -385,6 +395,15 @@ save_settings(CCCHHDoorIsDesktopWidget *self)
 		error = NULL;
 	}
 
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/conn_wlan_only", self->conn_wlan_only, &error))
+	{
+		g_warning(WIDGET_NAME": failed to set configuration value conn_wlan_only: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
 	if (!gconf_client_set_bool(self->gconf_client,
 		GCONF_DIR"/auto_update", self->auto_update, &error))
 	{
@@ -574,10 +593,12 @@ fetch_update_start (CCCHHDoorIsDesktopWidget *self)
 
 	g_debug(WIDGET_NAME": fetch_update_start()");
 
-	// TODO: fix conn_wlan_only logic here
-	//       for now, update anyway if force_banner is set
-	//       (indicating a button press)
-	if (!self->connected && !self->force_banner) {
+	// TODO: rethink conn_wlan_only logic here
+	//       for now, update anyway if force_banner is set,
+	//       indicating a button press, even if !connected.
+	//       self->connected does not always represent real
+	//       connection state if conn_wlan_only is set
+	if (!self->connected && !(self->conn_wlan_only && self->force_banner)) {
 		g_debug(WIDGET_NAME": not connected, not fetching new state");
 		return;
 	}
@@ -696,7 +717,7 @@ fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 
 	// first line of body is either "locked" or "unlocked"
 
-	if (!(line = strtok_r(self->gvfs_input_buffer, "\n", &tokctx))) {
+	if (!(line = strtok_r(buffer, "\n", &tokctx))) {
 		g_warning(WIDGET_NAME": first line of response not found");
 		fetch_update_process(self, new_state, new_change_ts, new_report_ts);
 		return;
@@ -840,6 +861,7 @@ fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
 		}
 	}
 
+	// set by button or state change
 	if (self->force_banner) {
 		self->force_banner = FALSE;
 		show_status_banner(self);
@@ -971,7 +993,7 @@ ccchh_dooris_desktop_widget_on_button_release_event (GtkWidget *widget,
 	//       will still see !connected. maybe wait a bit or use a timer event?
 	// TODO: conn_wlan_only: force libconic to use only WLAN IAPs.
 	//       for now, do nothing if conn_wlan_only is set
-	if (!self->conn_wlan_only && !self->connected && self->force_connection) {
+	if (!self->connected && self->force_connection && !self->conn_wlan_only) {
 		g_debug(WIDGET_NAME": not connected, requesting connection");
 		/* Request connection and check for the result */
 		gboolean success = con_ic_connection_connect(self->connection,
@@ -1009,6 +1031,7 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 	ConIcConnectionStatus status = con_ic_connection_event_get_status(event);
 	const gchar *iap_id = con_ic_event_get_iap_id(CON_IC_EVENT(event));
+	const gchar *bearer_id = con_ic_event_get_bearer_type(CON_IC_EVENT(event));
 
 	// Problem: When switching from GPRS to/from WLAN
 	// we receive STATUS_CONNECTED for the new IAP first,
@@ -1023,11 +1046,18 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 	switch (status) {
 		case CON_IC_STATUS_DISCONNECTING:
 
+			g_debug(WIDGET_NAME": connection event: disconnecting iap: %s", iap_id);
+
 			if (self->disconnecting_iap) {
 				g_free(self->disconnecting_iap);
 				self->disconnecting_iap = NULL;
 			}
 
+			if (self->connected_bearer) {
+				g_free(self->connected_bearer);
+				self->connected_bearer = NULL;
+			}
+
 			// save disconnecting IAP ID
 			if (iap_id)
 				self->disconnecting_iap = g_strdup(iap_id);
@@ -1040,12 +1070,23 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 		case CON_IC_STATUS_CONNECTED:
 
+			g_debug(WIDGET_NAME": connection event: connected bearer: %s", bearer_id);
+
 			// we are online now
 
+			if (self->connected_bearer) {
+				g_free(self->connected_bearer);
+				self->connected_bearer = NULL;
+			}
+
+			// save connected bearer
+			if (bearer_id)
+				self->connected_bearer = g_strdup(bearer_id);
+
 			// ignore online event if bearer type string does not begin
 			// with "WLAN_" and conn_wlan_only option is set
 			if (self->conn_wlan_only &&
-					strncmp(con_ic_event_get_bearer_type(CON_IC_EVENT(event)), "WLAN_", 5)) {
+					strncmp(self->connected_bearer, "WLAN_", 5)) {
 				g_debug(WIDGET_NAME": ignoring online event from non-WLAN IAP: conn_wlan_only set");
 
 			} else if (!self->connected) {
@@ -1066,10 +1107,12 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 			if (iap_id && self->disconnecting_iap && self->connected &&
 					!strcmp(iap_id, self->disconnecting_iap))
 			{
-				g_debug(WIDGET_NAME": ignoring offline event from disconnecting IAP");
+				g_debug(WIDGET_NAME": ignoring offline event from disconnecting IAP: %s", iap_id);
 
 			} else {
 
+				g_debug(WIDGET_NAME": connection event: disconnected IAP: %s", iap_id);
+
 				// we are offline now
 
 				self->connected = FALSE;
@@ -1092,6 +1135,8 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 		case CON_IC_STATUS_NETWORK_UP:
 
+			g_debug(WIDGET_NAME": connection event: network up");
+
 			// nothing to do here
 			break;
 	}
@@ -1117,6 +1162,7 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 
 			hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->notify_button), TRUE);
 			hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connforce_button), TRUE);
+			hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connwlan_button), FALSE);
 			hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connection_button), TRUE);
 
 			// TODO: avoid index constant
@@ -1182,7 +1228,6 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 
 				// force connection state to online
 				// redraw if necessary to remove disconnection overlay
-				// TODO: (maybe) fix conn_wlan_only logic here
 				if (!self->connected) {
 					self->connected = TRUE;
 					force_redraw(self);
@@ -1200,6 +1245,14 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 				g_debug(WIDGET_NAME": button does not force connection");
 			}
 
+			// use only WLAN connections
+			self->conn_wlan_only = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connwlan_button));
+			if (self->conn_wlan_only) {
+				g_debug(WIDGET_NAME": use only WLAN network");
+			} else {
+				g_debug(WIDGET_NAME": use any network");
+			}
+
 			save_settings(self);
 
 			// fall through to dispose dialog window
@@ -1217,12 +1270,14 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 			self->notify_button = NULL;
 			self->connection_button = NULL;
 			self->connforce_button = NULL;
+			self->connwlan_button = NULL;
 	}
 }
 
 
 // update interval picker callback - restrict value to UPDATE_INTERVAL_MIN - UPDATE_INTERVAL_MAX
-void update_picker_changed_value_cb (HildonPickerButton *picker, gpointer data)
+static void
+update_picker_changed_value_cb (HildonPickerButton *picker, gpointer data)
 {
 	gdouble number = 0;
 
@@ -1239,11 +1294,14 @@ void update_picker_changed_value_cb (HildonPickerButton *picker, gpointer data)
 
 
 // use_connection button callback - set force_connection (in-)sensitive
-void connection_button_toggled_cb (HildonCheckButton *button, gpointer data)
+static void
+connection_button_toggled_cb (HildonCheckButton *button, gpointer data)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET(data);
 	g_debug(WIDGET_NAME": connection_button_toggled_cb()");
-	gtk_widget_set_sensitive(self->connforce_button, hildon_check_button_get_active(button));
+	gboolean active = hildon_check_button_get_active(button);
+	gtk_widget_set_sensitive(self->connforce_button, active);
+	gtk_widget_set_sensitive(self->connwlan_button, active);
 }
 
 
@@ -1271,15 +1329,20 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 			G_CALLBACK (ccchh_dooris_desktop_widget_on_settings_dialog_response), 
 			self);
 
+		// set dialog size
+		// TODO: is this the right way?
+		gtk_window_set_default_size (GTK_WINDOW(self->settings_window), 410, 400);
+
 		// arrange the inputs
 		// we put controls and labels into hbox-es
-		// and stack them into the dialog's vbox
+		// and stack them into a vbox
 
-		GtkWidget *vbox = GTK_DIALOG(self->settings_window)->vbox;
-		gtk_container_set_border_width(GTK_CONTAINER (vbox), HILDON_MARGIN_DEFAULT);
+		GtkWidget *vbox = gtk_vbox_new (FALSE, HILDON_MARGIN_DEFAULT);
+//		GtkWidget *vbox = GTK_DIALOG(self->settings_window)->vbox;
+//		gtk_container_set_border_width(GTK_CONTAINER (vbox), HILDON_MARGIN_DEFAULT);
 
 		GtkWidget *hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 
 		GtkSizeGroup *sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 
@@ -1290,7 +1353,7 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_box_pack_start (GTK_BOX (hbox), caption, TRUE, TRUE, 0);
 
 		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 
 		gint selidx = 0;
 		gint idx = 1;
@@ -1351,7 +1414,7 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_box_pack_start (GTK_BOX (hbox), caption, TRUE, TRUE, 0);
 
 		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
     
 		// banner notifications
 		self->notify_button = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
@@ -1361,7 +1424,7 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_box_pack_start (GTK_BOX (hbox), self->notify_button, TRUE, TRUE, 0);
 
 		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 
 		// monitor internet connection, only update when online
 		self->connection_button = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
@@ -1369,11 +1432,11 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_button_set_alignment(GTK_BUTTON(self->connection_button), 0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox), self->connection_button, TRUE, TRUE, 0);
 
-		// install signal handler to toggle sensitivity of connforce_button
+		// install signal handler to toggle sensitivity of connforce/wlan_button
 		g_signal_connect (self->connection_button, "toggled", G_CALLBACK(connection_button_toggled_cb), self);
 
 		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 
 		// button forces connection
 		self->connforce_button = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
@@ -1381,6 +1444,24 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_button_set_alignment(GTK_BUTTON(self->connforce_button), 0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox), self->connforce_button, TRUE, TRUE, 0);
 
+		hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+
+		// use only WLAN
+		self->connwlan_button = hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT);
+		gtk_button_set_label(GTK_BUTTON (self->connwlan_button), _("Use only WLAN connections"));
+		gtk_button_set_alignment(GTK_BUTTON(self->connwlan_button), 0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox), self->connwlan_button, TRUE, TRUE, 0);
+
+		// pannable area provides scrolling
+		GtkWidget *pan = hildon_pannable_area_new ();
+
+		/* pack the filled vbox into the scrolled window */
+		hildon_pannable_area_add_with_viewport (HILDON_PANNABLE_AREA (pan), vbox);
+
+		/* Add the area into the window*/
+		gtk_box_pack_start (GTK_BOX(GTK_DIALOG(self->settings_window)->vbox), GTK_WIDGET(pan), TRUE, TRUE, 0);
+
 		g_object_unref(sizegroup);
 	}
 
@@ -1399,9 +1480,11 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->notify_button), self->notify_enabled);
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connection_button), self->use_connection);
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connforce_button), self->force_connection);
+	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connwlan_button), self->conn_wlan_only);
 	gtk_widget_set_sensitive(self->connforce_button, self->use_connection);
+	gtk_widget_set_sensitive(self->connwlan_button, self->use_connection);
 
-	gtk_widget_show_all (self->settings_window);
+	gtk_widget_show_all(self->settings_window);
 }
 
 
@@ -1492,6 +1575,11 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 		self->disconnecting_iap = NULL;
 	}
 
+	if (self->connected_bearer) {
+		g_free(self->connected_bearer);
+		self->connected_bearer = NULL;
+	}
+
 	if (self->gconf_client) {
 		g_object_unref(self->gconf_client);
 		self->gconf_client = NULL;
@@ -1574,6 +1662,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	self->force_banner = FALSE;
 
 	self->disconnecting_iap = NULL;
+	self->connected_bearer = NULL;
 
 	self->gvfs_handle = NULL;
 	self->gvfs_input_buffer = NULL;
