@@ -25,7 +25,7 @@
 #define TIMEOUT_CLOSE	15
 
 // max download size in bytes
-#define MAX_HTTP_SIZE 1024
+#define MAX_HTTP_SIZE 1023
 
 // minimum/maximum selectable update interval
 // in minutes
@@ -102,6 +102,7 @@ typedef struct {
 	gboolean highlighted;
 	gboolean too_old;
 	gchar *disconnecting_iap;
+	gchar *connected_bearer;
 	gdouble current_alpha;
 	GStaticMutex button_lock;
 	GStaticMutex fetch_lock;
@@ -130,6 +131,7 @@ typedef struct {
 	GtkWidget *notify_button;
 	GtkWidget *connection_button;
 	GtkWidget *connforce_button;
+	GtkWidget *connwlan_button;
 
 	// gvfs structures
 	GnomeVFSAsyncHandle *gvfs_handle;
